﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Windows.Forms;
using HeuristicLab.Hive.Contracts;
using HeuristicLab.Hive.Contracts.Interfaces;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Hive.Server.Core;

namespace HeuristicLab.Hive.Server {
  [Application("Hive Server", "Server application for the distributed hive engine.", false)]
  public class HeuristicLabHiveServerApplication : ApplicationBase {
    Dictionary<string, Uri> baseAddrDict = new Dictionary<string, Uri>();

    public override void Run() {
      ILifecycleManager lifecycleManager = null;
      ServiceHost slaveServiceHost = null;
      ServiceHost serverConsoleServiceHost = null;
      ServiceHost clientServiceHost = null;
      try {
        slaveServiceHost = new ServiceHost(ApplicationManager.Manager.GetTypes(typeof(ISlaveFacade)).First(), GetUri(WcfSettings.SlaveServiceName));
        slaveServiceHost.Open();

        serverConsoleServiceHost = new ServiceHost(ApplicationManager.Manager.GetTypes(typeof(IServerConsoleFacade)).First(), GetUri(WcfSettings.ServerConsoleServiceName));
        serverConsoleServiceHost.Open();

        clientServiceHost = new ServiceHost(ApplicationManager.Manager.GetTypes(typeof(IClientFacade)).First(), GetUri(WcfSettings.ClientServiceName));
        clientServiceHost.Open();

        lifecycleManager = ServiceLocator.GetLifecycleManager();
        lifecycleManager.Init();

        Form mainForm = new MainForm(baseAddrDict);
        Application.Run();
      }
      catch (AddressAccessDeniedException ex) {
        throw new Exception("Unable to start WCF-Services due to missing rights. Run the following command as administrator: \"netsh http add urlacl url=http://+" + WcfSettings.DefaultPort + "/ user=MYMACHINE\\UserName\". See inner exception for more details.", ex);
      }
      finally {
        if (slaveServiceHost != null && slaveServiceHost.State == CommunicationState.Opened)
          slaveServiceHost.Close();
        if (serverConsoleServiceHost != null && serverConsoleServiceHost.State == CommunicationState.Opened)
          serverConsoleServiceHost.Close();
        if (clientServiceHost != null && clientServiceHost.State == CommunicationState.Opened)
          clientServiceHost.Close();

        if(lifecycleManager != null)
          lifecycleManager.Shutdown();
      }
    }

    private Uri GetUri(string serviceName) {
      string ipAddress = WcfSettings.GetActiveIP().ToString();
      string port = WcfSettings.DefaultPort.ToString();
      Uri uriHttp = new Uri("http://" + ipAddress + ":" + port + "/" + serviceName + "/");
      baseAddrDict.Add(serviceName, uriHttp);
      return uriHttp;
    }

  }
}
